#include "AudioCore.h"

AudioCore::AudioCore()
{
	fprintf(stderr, "initing audiocore\n");
}

AudioCore::~AudioCore()
{
	fprintf(stderr, "killing audiocore\n");
	UnloadWAV();
	SDL_PauseAudio(1);
	SDL_CloseAudio();
}

void AudioCore::SetupCore(int freq, int ch)
{
	SDL_AudioSpec acfmt;

	acfmt.freq = freq;
	acfmt.channels = ch;
	acfmt.format = AUDIO_S16SYS;
	acfmt.samples = 2048;
	acfmt.callback = AudioCore::audio_callback_trampoline;
	acfmt.userdata = this;
	
	SDL_OpenAudio(&acfmt, NULL);
	sample_pos = NULL;

	SDL_PauseAudio(0);
}

void AudioCore::audio_callback(Uint8 *stream, int len)
{
	if(sample_pos == NULL) {
		memset(stream, 0, len);
	} else if(sample_pos + len > sample + sample_len) {
		int dleft = sample_len - (sample_pos - sample);
		memcpy(stream, sample_pos, dleft);
		memset(stream+dleft, 0, len-dleft);
		sample_pos = NULL;
	} else {
		memcpy(stream, sample_pos, len);
		sample_pos += len;
	}
}

void AudioCore::LoadSingleWAV(string path)
{
	SDL_AudioSpec spc, *got;
	spc.format = AUDIO_S16SYS;
	spc.channels = 2;
	spc.freq = 44100;
	Uint8 *ins; Uint32 sl;

	got = SDL_LoadWAV(path.c_str(), &spc, &ins, &sl);
	if(got) {
		// convert to my format
		SDL_AudioCVT cvt;
		if(SDL_BuildAudioCVT(&cvt, got->format, got->channels, got->freq, AUDIO_S16SYS, 2, 44100) == 1) {
			sample = (unsigned char *)malloc(cvt.len_mult * sl);
			memcpy(sample, ins, sl);
			cvt.buf = sample;
			cvt.len = sl;
			if(SDL_ConvertAudio(&cvt) == 0) {
				sample_len = (long)(cvt.len_ratio * sl);
			} else {
				free(sample);
				sample_len = 0;
				sample = NULL;
				fprintf(stderr, "Failed to convert %s\n", path.c_str());
			}
		}

		SDL_FreeWAV(ins);
	} else {
		fprintf(stderr, "Failed to load %s\n", path.c_str());
	}
}

void AudioCore::UnloadWAV()
{
	if(sample) {
		free(sample);
		sample_pos = sample = NULL;
		sample_len = 0;
	}
}

void AudioCore::Play()
{
	sample_pos = sample;
}

