#ifndef __Robot_h__
#define __Robot_h__

#include "Object.h"

using namespace std;

enum {
	ROBOT_GEOM_UNKNOWN,
	ROBOT_GEOM_BODY,
	ROBOT_GEOM_FREEWHEEL,
	ROBOT_GEOM_DRIVEWHEEL,
};

class Robot : public Object {
public:
	Robot();
	virtual ~Robot();
	virtual int GetType() { return OBJECT_ROBOT; }
	virtual void Draw();
	virtual void SetupODE(dWorldID, dSpaceID);
	virtual void UpdatePositions();
	virtual void PrepareForCollision();
	virtual int HandleCollision(dGeomID you, int wp, Object *other, dSurfaceParameters &surface);
	virtual void PostCollision();

	void ToggleTrail() {
		enable_trail = !enable_trail;
	}

	int GetGeomType(dGeomID d) {
		if(d == geoms[0]) return ROBOT_GEOM_BODY;
		if(d == geoms[1]) return ROBOT_GEOM_FREEWHEEL;
		if(d == geoms[2] || d == geoms[3]) return ROBOT_GEOM_DRIVEWHEEL;
		fprintf(stderr, "unknown geom?\n");
		return ROBOT_GEOM_UNKNOWN;
	}

	double GetLeftActualVel();
	double GetRightActualVel();
	void SetLeftWheelVel(double rps);
	void SetRightWheelVel(double rps);

	long GetLeftEncoder() { return left_encoder; }
	long GetRightEncoder() { return right_encoder; }
	double GetLeftCommandedVel() { return lm_cmdvel; }
	double GetRightCommandedVel() { return rm_cmdvel; }
	double GetAxleLength() { return axis_length; }
	double GetWheelRadius() { return wheel_radius; }
	long GetTicksPerRevolution() { return (long)ticks_per_revolution; }

	bool GetCollision(int &collision) {
		lock_rc_m();
		if(recent_collisions.size()) {
			collision = recent_collisions.front();
			recent_collisions.pop();
			unlock_rc_m();
			return true;
		}
		unlock_rc_m();
		return false;
	}

private:

	// splinter bot
	dBodyID  bodies[4];
	dJointID wheel_joints[3];
	dSpaceID robot_space;

	double llan, lran;
	long int left_encoder;
	long int right_encoder;

	double ticks_per_revolution;

	double wheel_radius;
	double axis_length;
	bool first_encoder_read;

	double lm_cmdvel, rm_cmdvel;

	bool enable_trail;
	map<double,pair<double, double> > history;
	SDL_mutex *history_m;
	void lock_history() { SDL_mutexP(history_m); }
	void unlock_history() { SDL_mutexV(history_m); }
	double maxage;
	double history_per_sec;

	void CleanupHistory(double maxage);

	double max_wheel_revs_per_sec;

	queue<int> recent_collisions;
	SDL_mutex *rc_m;
	void lock_rc_m() { SDL_mutexP(rc_m); }
	void unlock_rc_m() { SDL_mutexV(rc_m); }

	dReal last_pos[4];
	dReal last_quat[4];
};

#endif
