#include "UCInterface.h"
#include "dtime.h"
#include <unistd.h>

UCInterface::UCInterface(double stepsize, string ucparams, string logfile, SimCore *sc)
{
	this->stepsize = stepsize;
	this->logfile = logfile;
	this->sc = sc;
	this->ucparams = ucparams;

	uc_running = false;
	
	uc_mtx = SDL_CreateMutex();

	pause = true;

	wc = WorldClock::WC();
	
	uc_thread = SDL_CreateThread(trampoline, this);
}

UCInterface::~UCInterface()
{
	uc_running = false;
	SDL_WaitThread(uc_thread, NULL);
}

void UCInterface::collision(int type)
{
	lock();
	collisions.push(type);
	unlock();
}

void UCInterface::key_press(int key)
{
	lock();
	keypresses.push(key);
	unlock();
}

void UCInterface::mouse_click(int x, int y, double wx, double wy, int button)
{
	click_t t;
	t.x = x; t.y = y; t.wx = wx; t.wy = wy; t.button = button;
	lock();
	clicks.push(t);
	unlock();
}

void UCInterface::UserCodeIFRun()
{
	double lwc, n;

	rif = new RobotIF(sc);
	user = new UserCode(*rif);

	ParseParams();
	user->setup();

	// start logging
	int nv = 4 + user->num_log_vars();
	logger = new DataLogger(logfile, nv);

	double *lv = (double *)malloc(nv * sizeof(double));

	uc_running = true;
	while(uc_running) {
		lwc = wc->BeginTimedCycle();

		lock();
		/* call usercode */
		while(keypresses.size()) {
			user->key_press(keypresses.front());
			keypresses.pop();
		}
		while(collisions.size()) {
			user->collision(collisions.front());
			collisions.pop();
		}
		while(clicks.size()) {
			click_t t = clicks.front();
			user->mouse_click(t.x, t.y, t.wx, t.wy, t.button);
			clicks.pop();
		}
		user->cycle();
		unlock();

		/* get log var stuff */
		sc->GetRobotPosVec(lv);
		user->get_log_data(lv+4);
		logger->Logv(0, nv, lv);
		
		wc->DelayTime(stepsize, lwc);
	}

	free(lv);

	delete user;
	delete rif;
	delete logger;
}


void UCInterface::ParseParams()
{
	char *s = strdup(ucparams.c_str());
	char *op = s, *p = s;

	bool run = false;
	
	while((p = strchr(p, ',')) != NULL) {
		run = true;
		*p = 0;
		ParseArg(op);
		op = ++p;
	}

	ParseArg(op);
	
	free(s);
}

void UCInterface::ParseArg(char *a)
{
	if(a == NULL) return;
	if(*a == 0) return;

	char *key = a, *val;
	if((val = strchr(a, '=')) != NULL) *val++ = 0;
	if(val == NULL) val = "";

	user->handle_param(string(key), string(val));
}

