#ifndef __UCInterface_h__
#define __UCInterface_h__

#include "core.h"
#include "DataLogger.h"
#include "RobotIF.h"
#include "user_code.h"
#include "WorldClock.h"
#include "SDL/SDL.h"
#include "SDL/SDL_thread.h"

#include <queue>

class SimCore;
class RobotIF;
class UserCode;

typedef struct {
	int x, y;
	double wx, wy;
	int button;
} click_t;

class UCInterface {
public:
	UCInterface(double stepsize, string ucparams, string logfile, SimCore *sc);
	~UCInterface();

	void Pause() { pause = true; }
	void Resume() { pause = false; }
	bool Paused() { return pause; }
	bool Ready() { return uc_running; }

	void collision(int);
	void key_press(int);
	void mouse_click(int x, int y, double wx, double wy, int button);

private:
	
	double stepsize;

	// ...
	SDL_Thread *uc_thread;
	SDL_mutex *uc_mtx;
	static int trampoline(void *t) { ((UCInterface *)t)->UserCodeIFRun(); return 0; }
	void UserCodeIFRun();

	void lock() { SDL_mutexP(uc_mtx); }
	void unlock() { SDL_mutexV(uc_mtx); }
	
	bool uc_running;
	bool pause;

	WorldClock *wc;

	SimCore *sc;
	string logfile;
	RobotIF *rif;
	UserCode *user;
	DataLogger *logger;
	string ucparams;

	void ParseParams();
	void ParseArg(char *p);

	queue<int> keypresses;
	queue<int> collisions;
	queue<click_t> clicks;
};

#endif
